@@ -1,5 +1,5 @@
 #
-# Copyright [1993-2016] [Charles Kooperberg]
+# Copyright [1993-2018] [Charles Kooperberg]
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -48,61 +48,25 @@ oldlogspline.to.logspline <- function(obj,data)
    nobj$logl <- obj$logl[obj$logl!=0]
    lx <- length(nobj$logl)
    nobj$logl <- cbind(nobj$maxknots+1-(lx:1),c(rep(2,lx-1),1),nobj$logl)
+   class(nobj) <- "logspline"
    if(!missing(data))nobj$range <- obj$range
    else {
       lx <- 1/(nobj$samples+1)
-      nobj$range <- qoldlogspline(c(lx,1-lx),obj)
+      nobj$range <- qlogspline(c(lx,1-lx),nobj)
    }
    nobj$mind
-   class(nobj) <- "logspline"
    nobj
 }
 poldlogspline <- function(q, fit)
 {
-    if(class(fit)!="oldlogspline")
-       stop("fit is not an oldlogspline object")
-        q <- unstrip(q)
-    sq <- rank(q)
-    q <- sort(q)
-    z <- .C("pqlsd",
-        as.double(fit$coef),
-        as.double(fit$knots),
-        as.double(fit$bound),
-        as.integer(1),
-        pp = as.double(q),
-        as.double(q),
-        as.integer(length(fit$knots)),
-        as.integer(length(q)),
-        PACKAGE = "logspline")
-    zz <- z$pp[sq]
-    if(fit$bound[1] > 0)
-        zz[q<fit$bound[2]] <- 0
-    if(fit$bound[3] > 0)
-        zz[q>fit$bound[4]] <- 1
-    zz
+    fitx <- oldlogspline.to.logspline(fit)
+    plogspline(q,fitx)
 }
 
 qoldlogspline <- function(p, fit)
 {
-    if(class(fit)!="oldlogspline")
-       stop("fit is not an oldlogspline object")
-        p <- unstrip(p)
-    sp <- rank(p)
-    p <- sort(p)
-    z <- .C("pqlsd",
-        as.double(fit$coef),
-        as.double(fit$knots),
-        as.double(fit$bound),
-        as.integer(0),
-        as.double(p),
-        qq = as.double(p),
-        as.integer(length(fit$knots)),
-        as.integer(length(p)),
-        PACKAGE = "logspline")
-    zz <- z$qq[sp]
-    zz[p<0] <- NA
-    zz[p>1] <- NA
-    zz
+    fitx <- oldlogspline.to.logspline(fit)
+    qlogspline(p,fitx)
 }
 
 roldlogspline <- function(n, fit)
@@ -352,7 +316,9 @@ oldlogspline <- function(uncensored, right, left, interval, lbound, ubound,
 # SorC will carry the error messages - in code form
         SorC <- vector(mode = "integer", length = 35)
         SorC[1] <- 1    # the actual function call
+        SorC[17] <- 0
         nsample[6] <- nsample[6]-1
+        if(length(table(sample))<3)stop("Not enough unique values")
         z <- .C("logcensor",
                 as.integer(delete),
                 as.integer(iautoknot),
@@ -368,6 +334,27 @@ oldlogspline <- function(uncensored, right, left, interval, lbound, ubound,
                 as.double(sample),
                 logl = as.double(rep(0, n1 + 1)),
         PACKAGE = "logspline")
+        SorC <- z$SorC  # error messages
+        if(SorC[1] == -1 && SorC[28] == 0 && nsample[1]!=nsample[2] && nsample[2]>15){
+           SorC <- vector(mode = "integer", length = 35)
+           SorC[1] <- 1    # the actual function call
+           SorC[17] <- 1
+           z <- .C("logcensor",
+                   as.integer(delete),
+                   as.integer(iautoknot),
+                   as.double(sample),
+                   as.integer(nsample),
+                   bd = as.double(xbound),
+                   SorC = as.integer(SorC),
+                   nk = as.integer(nknots),
+                   kt = as.double(knots),
+                   cf = as.double(c(knots, 0, 0)),
+                   as.double(penalty),
+                   as.double(sample),
+                   as.double(sample),
+                   logl = as.double(rep(0, n1 + 1)),
+           PACKAGE = "logspline")
+        }
         bound <- c(z$bd[2], z$bd[3], z$bd[4], z$bd[5])
         SorC <- z$SorC  # error messages
         if(abs(SorC[1]) > 2) {
@@ -438,6 +425,7 @@ logspline <- function(x, lbound, ubound, maxknots=0, knots, nknots=0,
    call <- match.call()
    if(!missing(x))x <- unstrip(x)
    data <- x
+   if(length(table(data))<3)stop("Not enough unique values")
    ilx <- 0; iux <- 0
    if(!missing(lbound)){ilx <- 1;jlx <- lbound}
    if(!missing(ubound)){iux <- 1;jux <- ubound}
@@ -583,7 +571,7 @@ plogspline <- function(q, fit)
     zz <- z$pp[sq]
     if(fit$bound[1] > 0) zz[q<fit$bound[2]] <- 0
     if(fit$bound[3] > 0) zz[q>fit$bound[4]] <- 1
-    zz
+    zz 
 }
 qlogspline <- function(p, fit)
 {
